{

  Copyright 1994 by Salvatore Besso, mc8505@mclink.it

  This software is freeware.

  You are free to modify the source code for your personal use and to
  redistribute this software only if you leave the copyright notices
  unmodified everywhere in the code, also in the comments.

  In case of problems contact me via e-mail at address:

  mc8505@mclink.it

}

program TablesDescription; { Prints Paradox 4 tables structure }

{
  Errorlevels returned:

  1  = File error (not found or other DOS error)
  2  = Paradox Engine error
  3  = Printer error
  99 = Command line parameter missing
}

uses

  Dos,Printer,

  Drivers,Objects,

  PXEngine,OOPxEng;

const
  Engine  : PEngine   = NIL;
  DataBase: PDataBase = NIL;

var
  Path: PathStr;

procedure ReleaseEngine;

begin
  if Database <> NIL then Dispose (Database,Done);
  if Engine <> NIL then Dispose (Engine,Done)
end;

procedure ShutDown (Code: RetCode);

begin
  PrintStr (#7#13#10'Paradox Engine Database Framework error:'#13#10);
  PrintStr (Engine^.GetErrorMessage (Code) + #13#10);
  ReleaseEngine;
  Halt (2)
end;

procedure OpenEngine;

begin
  Engine := New (PEngine,defInit (pxLocal));
  if Engine^.LastError <> PXSuccess then
    ShutDown (Engine^.LastError);
  DataBase := New (PDataBase,Init (Engine));
  if DataBase^.LastError <> PXSuccess then
    ShutDown (DataBase^.LastError)
end;

procedure PrinterError;

begin
  PrintStr (#7#13#10'Printer error'#13#10);
  ReleaseEngine;
  Halt (3)
end;

procedure PrintStructure;

var
  DirInfo        : SearchRec;
  D              : DirStr;
  N              : NameStr;
  E              : ExtStr;
  Line,Spaces    : String;
  TableDescriptor: PTableDesc;

procedure PrintInfo (P: PFieldDesc); far;

var
  S,T     : String;
  Len,Size: Integer;

begin
  Str (P^.FldNum:0,T);
  S := Copy (T + Spaces,1,3) + ': ' + Copy (P^.FldName + Spaces,1,26);
  case P^.FldType of
    fldChar  : T := 'Alpha';
    fldShort : T := 'Short';
    fldDate  : T := 'Date';
    fldDouble: T := 'Number';
    fldBlob  : T := 'Blob'
  end;
  S := S + Copy (T + Spaces,1,10);
  case P^.FldSubType of
    fldStNone   : T := 'None';
    fldStMoney  : T := 'Currency';
    fldStMemo   : T := 'Memo';
    fldStBinary : T := 'Binary';
    fldStFmtMemo: T := 'Fmt Memo';
    fldStOleObj : T := 'OLE';
    fldStGraphic: T := 'Graphic'
  end;
  S := S + Copy (T + Spaces,1,10);
  case P^.FldType of
    fldChar  : Len := P^.FldLen;
    fldShort : Len := 2;
    fldDate  : Len := 4;
    fldDouble: Len := 8;
    fldBlob  : Len := P^.FldLen
  end;
  Str (Len:0,T);
  S := S + Copy (T + Spaces,1,7);
  Size := Len;
  if P^.FldType = fldBlob then Inc (Size,10);
  Str (Size:0,T);
  S := S + T;
  {$I-}
  WriteLn (Lst,S);
  {$I+}
  if IOResult > 0 then PrinterError
end;

begin { PrintStructure }
  FillChar (Line[1],80,'-');
  Line[0] := #80;
  FillChar (Spaces[1],80,' ');
  Spaces[0] := #80;
  FSplit (FExpand (Path),D,N,E);
  if N = '' then N := '*';
  Path := D + N + '.DB';
  FindFirst (Path,Archive,DirInfo);
  if DosError > 0 then
  begin
    PrintStr (#7#13#10'File not found or file error'#13#10);
    ReleaseEngine;
    Halt (1)
  end;
  while DosError = 0 do
  begin
    N := Copy (DirInfo.Name,1,Pos ('.',DirInfo.Name) - 1);
    if IOResult > 0 then PrinterError;
    TableDescriptor := DataBase^.GetDescVector (D + N);
    if DataBase^.LastError <> PXSuccess then
      ShutDown (DataBase^.LastError);
    {$I-}
    WriteLn (Lst);
    WriteLn (Lst,Copy (Spaces,1,(80 - Byte (N[0])) div 2),N);
    WriteLn (Lst,Line);
    WriteLn (Lst,'#    Description               Type      Sub       Len',
      '    Size');
    WriteLn (Lst,Line);
    {$I+}
    if IOResult > 0 then PrinterError;
    TableDescriptor^.ForEach (@PrintInfo);
    Dispose (TableDescriptor,Done);
    {$I-}
    Write (Lst,#12#13);
    {$I+}
    if IOResult > 0 then PrinterError;
    FindNext (DirInfo)
  end
end;

begin { Main }
  PrintStr ('TablDesc 1.00 - Prints Paradox 4 tables structure'#13#10 +
    'by Salvatore Besso, mc8505@mclink.it'#13#10);
  if ParamCount = 0 then
  begin
    PrintStr (#13#10'Use: TablDesc [drive:\path\]tablename[.DB]' +
      '  (wildcards are OK)'#13#10);
    Halt (99)
  end
  else Path := ParamStr (1);
  OpenEngine;
  PrintStructure;
  ReleaseEngine
end.
